//--------------------------------------------------------------
// File     : stm32_ub_ext_int5to9.c
// Datum    : 01.04.2013
// Version  : 1.0
// Autor    : UB
// EMail    : mc-4u(@)t-online.de
// Web      : www.mikrocontroller-4u.de
// CPU      : STM32F4
// IDE      : CooCox CoIDE 1.7.0
// Module   : GPIO, EXTI, SYSCFG, MISC
// Funktion : External Interrupt5 bis 9
//
// Hinweis  : mgliche Pinbelegungen
//            EXT_INT5 :
//               [PA5, PB5, PC5, PD5, PE5, PF5, PG5, PH5, PI5]
//            EXT_INT6 :
//               [PA6, PB6, PC6, PD6, PE6, PF6, PG6, PH6, PI6]
//            EXT_INT7 :
//               [PA7, PB7, PC7, PD7, PE7, PF7, PG7, PH7, PI7]
//            EXT_INT8 :
//               [PA8, PB8, PC8, PD8, PE8, PF8, PG8, PH8, PI8]
//            EXT_INT9 :
//               [PA9, PB9, PC9, PD9, PE9, PF9, PG9, PH9, PI9]
//--------------------------------------------------------------


//--------------------------------------------------------------
// Includes
//--------------------------------------------------------------
#include "stm32_ub_ext_int5to9.h"
// ab hier eigene includes



//--------------------------------------------------------------
// interne Funktionen
//--------------------------------------------------------------
void P_EXT_INT5_LoFlanke(void);
void P_EXT_INT5_HiFlanke(void);
void P_EXT_INT6_LoFlanke(void);
void P_EXT_INT6_HiFlanke(void);
void P_EXT_INT7_LoFlanke(void);
void P_EXT_INT7_HiFlanke(void);
void P_EXT_INT8_LoFlanke(void);
void P_EXT_INT8_HiFlanke(void);
void P_EXT_INT9_LoFlanke(void);
void P_EXT_INT9_HiFlanke(void);



//--------------------------------------------------------------
// Definition vom EXT_INT5 bis 9 Pin
//
// Widerstand : [GPIO_PuPd_NOPULL,GPIO_PuPd_UP,GPIO_PuPd_DOWN]
// Trigger    : [EXTI_Trigger_Rising,EXTI_Trigger_Falling,EXTI_Trigger_Rising_Falling]
//--------------------------------------------------------------
EXT_INT5TO9_t EXT_INT5TO9[] = {
//NR , PORT, PIN      , CLOCK              ,Source,             ,Widerstand      , Trigger
  {8 ,GPIOA,GPIO_Pin_8,RCC_AHB1Periph_GPIOA,EXTI_PortSourceGPIOA,GPIO_PuPd_NOPULL,EXTI_Trigger_Rising_Falling},   // EXT_Int-Pin = PA8
  {7 ,GPIOB,GPIO_Pin_7,RCC_AHB1Periph_GPIOB,EXTI_PortSourceGPIOB,GPIO_PuPd_NOPULL,EXTI_Trigger_Rising_Falling},   // EXT_Int-Pin = PB7
};



//--------------------------------------------------------------
// Init vom external Interrupt 5 bis 9
//--------------------------------------------------------------
void UB_Ext_INT5TO9_Init(void)
{
  GPIO_InitTypeDef   GPIO_InitStructure;
  EXTI_InitTypeDef   EXTI_InitStructure;
  NVIC_InitTypeDef   NVIC_InitStructure;
  uint8_t nr;

  for(nr=0;nr<EXT_INT5TO9_ANZ;nr++) {
    // Clock enable (GPIO)
    RCC_AHB1PeriphClockCmd(EXT_INT5TO9[nr].INT_CLK, ENABLE);

    // Config als Digital-Eingang
    GPIO_InitStructure.GPIO_Mode = GPIO_Mode_IN;
    GPIO_InitStructure.GPIO_PuPd = EXT_INT5TO9[nr].INT_R;
    GPIO_InitStructure.GPIO_Pin = EXT_INT5TO9[nr].INT_PIN;
    GPIO_Init(EXT_INT5TO9[nr].INT_PORT, &GPIO_InitStructure);
    
    // Clock enable (SYSCONFIG)
    RCC_APB2PeriphClockCmd(RCC_APB2Periph_SYSCFG, ENABLE);

    if(EXT_INT5TO9[nr].INT_NR==5) {
      // EXT_INT5 mit Pin verbinden
      SYSCFG_EXTILineConfig(EXT_INT5TO9[nr].INT_SOURCE, EXTI_PinSource5);

      // EXT_INT5 config
      EXTI_InitStructure.EXTI_Line = EXTI_Line5;
      EXTI_InitStructure.EXTI_Mode = EXTI_Mode_Interrupt;
      EXTI_InitStructure.EXTI_Trigger = EXT_INT5TO9[nr].INT_TRG;
      EXTI_InitStructure.EXTI_LineCmd = ENABLE;
      EXTI_Init(&EXTI_InitStructure);
    }
    else if(EXT_INT5TO9[nr].INT_NR==6) {
      // EXT_INT6 mit Pin verbinden
      SYSCFG_EXTILineConfig(EXT_INT5TO9[nr].INT_SOURCE, EXTI_PinSource6);

      // EXT_INT6 config
      EXTI_InitStructure.EXTI_Line = EXTI_Line6;
      EXTI_InitStructure.EXTI_Mode = EXTI_Mode_Interrupt;
      EXTI_InitStructure.EXTI_Trigger = EXT_INT5TO9[nr].INT_TRG;
      EXTI_InitStructure.EXTI_LineCmd = ENABLE;
      EXTI_Init(&EXTI_InitStructure);
    }
    else if(EXT_INT5TO9[nr].INT_NR==7) {
      // EXT_INT7 mit Pin verbinden
      SYSCFG_EXTILineConfig(EXT_INT5TO9[nr].INT_SOURCE, EXTI_PinSource7);

      // EXT_INT7 config
      EXTI_InitStructure.EXTI_Line = EXTI_Line7;
      EXTI_InitStructure.EXTI_Mode = EXTI_Mode_Interrupt;
      EXTI_InitStructure.EXTI_Trigger = EXT_INT5TO9[nr].INT_TRG;
      EXTI_InitStructure.EXTI_LineCmd = ENABLE;
      EXTI_Init(&EXTI_InitStructure);
    }
    else if(EXT_INT5TO9[nr].INT_NR==8) {
      // EXT_INT8 mit Pin verbinden
      SYSCFG_EXTILineConfig(EXT_INT5TO9[nr].INT_SOURCE, EXTI_PinSource8);

      // EXT_INT8 config
      EXTI_InitStructure.EXTI_Line = EXTI_Line8;
      EXTI_InitStructure.EXTI_Mode = EXTI_Mode_Interrupt;
      EXTI_InitStructure.EXTI_Trigger = EXT_INT5TO9[nr].INT_TRG;
      EXTI_InitStructure.EXTI_LineCmd = ENABLE;
      EXTI_Init(&EXTI_InitStructure);
    }
    else if(EXT_INT5TO9[nr].INT_NR==9) {
      // EXT_INT9 mit Pin verbinden
      SYSCFG_EXTILineConfig(EXT_INT5TO9[nr].INT_SOURCE, EXTI_PinSource9);

      // EXT_INT9 config
      EXTI_InitStructure.EXTI_Line = EXTI_Line9;
      EXTI_InitStructure.EXTI_Mode = EXTI_Mode_Interrupt;
      EXTI_InitStructure.EXTI_Trigger = EXT_INT5TO9[nr].INT_TRG;
      EXTI_InitStructure.EXTI_LineCmd = ENABLE;
      EXTI_Init(&EXTI_InitStructure);
    }
  }

  // NVIC config
  NVIC_InitStructure.NVIC_IRQChannel = EXTI9_5_IRQn;
  NVIC_InitStructure.NVIC_IRQChannelPreemptionPriority = 0x01;
  NVIC_InitStructure.NVIC_IRQChannelSubPriority = 0x01;
  NVIC_InitStructure.NVIC_IRQChannelCmd = ENABLE;
  NVIC_Init(&NVIC_InitStructure);
}


//--------------------------------------------------------------
// diese Funktion wird aufgerufen,
// bei einer Lo-Flanke am EXT-INT5
// wenn [EXTI_Trigger_Falling oder EXTI_Trigger_Rising_Falling]
//--------------------------------------------------------------
void P_EXT_INT5_LoFlanke(void)
{
  // hier eigenen Code eintragen

}


//--------------------------------------------------------------
// diese Funktion wird aufgerufen,
// bei einer Hi-Flanke am EXT-INT5
// wenn [EXTI_Trigger_Rising oder EXTI_Trigger_Rising_Falling]
//--------------------------------------------------------------
void P_EXT_INT5_HiFlanke(void)
{
  // hier eigenen Code eintragen

}


//--------------------------------------------------------------
// diese Funktion wird aufgerufen,
// bei einer Lo-Flanke am EXT-INT6
// wenn [EXTI_Trigger_Falling oder EXTI_Trigger_Rising_Falling]
//--------------------------------------------------------------
void P_EXT_INT6_LoFlanke(void)
{
  // hier eigenen Code eintragen

}


//--------------------------------------------------------------
// diese Funktion wird aufgerufen,
// bei einer Hi-Flanke am EXT-INT6
// wenn [EXTI_Trigger_Rising oder EXTI_Trigger_Rising_Falling]
//--------------------------------------------------------------
void P_EXT_INT6_HiFlanke(void)
{
  // hier eigenen Code eintragen

}


//--------------------------------------------------------------
// diese Funktion wird aufgerufen,
// bei einer Lo-Flanke am EXT-INT7
// wenn [EXTI_Trigger_Falling oder EXTI_Trigger_Rising_Falling]
//--------------------------------------------------------------
void P_EXT_INT7_LoFlanke(void)
{
  // hier eigenen Code eintragen

}


//--------------------------------------------------------------
// diese Funktion wird aufgerufen,
// bei einer Hi-Flanke am EXT-INT7
// wenn [EXTI_Trigger_Rising oder EXTI_Trigger_Rising_Falling]
//--------------------------------------------------------------
void P_EXT_INT7_HiFlanke(void)
{
  // hier eigenen Code eintragen

}


//--------------------------------------------------------------
// diese Funktion wird aufgerufen,
// bei einer Lo-Flanke am EXT-INT8
// wenn [EXTI_Trigger_Falling oder EXTI_Trigger_Rising_Falling]
//--------------------------------------------------------------
void P_EXT_INT8_LoFlanke(void)
{
  // hier eigenen Code eintragen

}


//--------------------------------------------------------------
// diese Funktion wird aufgerufen,
// bei einer Hi-Flanke am EXT-INT8
// wenn [EXTI_Trigger_Rising oder EXTI_Trigger_Rising_Falling]
//--------------------------------------------------------------
void P_EXT_INT8_HiFlanke(void)
{
  // hier eigenen Code eintragen

}


//--------------------------------------------------------------
// diese Funktion wird aufgerufen,
// bei einer Lo-Flanke am EXT-INT9
// wenn [EXTI_Trigger_Falling oder EXTI_Trigger_Rising_Falling]
//--------------------------------------------------------------
void P_EXT_INT9_LoFlanke(void)
{
  // hier eigenen Code eintragen

}


//--------------------------------------------------------------
// diese Funktion wird aufgerufen,
// bei einer Hi-Flanke am EXT-INT9
// wenn [EXTI_Trigger_Rising oder EXTI_Trigger_Rising_Falling]
//--------------------------------------------------------------
void P_EXT_INT9_HiFlanke(void)
{
  // hier eigenen Code eintragen

}


//--------------------------------------------------------------
// external Interrupt-5 bis 9
// wird je nach einstellung der Triggerflanke
// entweder bei Hi- oder Lo- oder beiden Flanken ausgelst
//--------------------------------------------------------------
void EXTI9_5_IRQHandler(void)
{
  uint32_t wert;
  uint8_t nr;

  if(EXTI_GetITStatus(EXTI_Line5) != RESET)
  {
    // wenn Interrupt aufgetreten
    EXTI_ClearITPendingBit(EXTI_Line5);

    for(nr=0;nr<EXT_INT5TO9_ANZ;nr++) {
      if(EXT_INT5TO9[nr].INT_NR==5) {
        wert=GPIO_ReadInputDataBit(EXT_INT5TO9[nr].INT_PORT, EXT_INT5TO9[nr].INT_PIN);
        if(wert==Bit_RESET) {
          P_EXT_INT5_LoFlanke();
        }
        else {
          P_EXT_INT5_HiFlanke();
        }
        break;        
      }
    }
  }
  if(EXTI_GetITStatus(EXTI_Line6) != RESET)
  {
    // wenn Interrupt aufgetreten
    EXTI_ClearITPendingBit(EXTI_Line6);

    for(nr=0;nr<EXT_INT5TO9_ANZ;nr++) {
      if(EXT_INT5TO9[nr].INT_NR==6) {
        wert=GPIO_ReadInputDataBit(EXT_INT5TO9[nr].INT_PORT, EXT_INT5TO9[nr].INT_PIN);
        if(wert==Bit_RESET) {
          P_EXT_INT6_LoFlanke();
        }
        else {
          P_EXT_INT6_HiFlanke();
        }
        break;        
      }
    }
  }
  if(EXTI_GetITStatus(EXTI_Line7) != RESET)
  {
    // wenn Interrupt aufgetreten
    EXTI_ClearITPendingBit(EXTI_Line7);

    for(nr=0;nr<EXT_INT5TO9_ANZ;nr++) {
      if(EXT_INT5TO9[nr].INT_NR==7) {
        wert=GPIO_ReadInputDataBit(EXT_INT5TO9[nr].INT_PORT, EXT_INT5TO9[nr].INT_PIN);
        if(wert==Bit_RESET) {
          P_EXT_INT7_LoFlanke();
        }
        else {
          P_EXT_INT7_HiFlanke();
        }
        break;        
      }
    }
  }
  if(EXTI_GetITStatus(EXTI_Line8) != RESET)
  {
    // wenn Interrupt aufgetreten
    EXTI_ClearITPendingBit(EXTI_Line8);

    for(nr=0;nr<EXT_INT5TO9_ANZ;nr++) {
      if(EXT_INT5TO9[nr].INT_NR==8) {
        wert=GPIO_ReadInputDataBit(EXT_INT5TO9[nr].INT_PORT, EXT_INT5TO9[nr].INT_PIN);
        if(wert==Bit_RESET) {
          P_EXT_INT8_LoFlanke();
        }
        else {
          P_EXT_INT8_HiFlanke();
        }
        break;        
      }
    }
  }
  if(EXTI_GetITStatus(EXTI_Line9) != RESET)
  {
    // wenn Interrupt aufgetreten
    EXTI_ClearITPendingBit(EXTI_Line9);

    for(nr=0;nr<EXT_INT5TO9_ANZ;nr++) {
      if(EXT_INT5TO9[nr].INT_NR==9) {
        wert=GPIO_ReadInputDataBit(EXT_INT5TO9[nr].INT_PORT, EXT_INT5TO9[nr].INT_PIN);
        if(wert==Bit_RESET) {
          P_EXT_INT9_LoFlanke();
        }
        else {
          P_EXT_INT9_HiFlanke();
        }
        break;        
      }
    }
  }
}

